var VueFlowchart = (function (exports) {
  'use strict';

  const DEFAULT_NODE = {
    id: null,
    title: "",
    bodies: [''],
    x: 0,
    y: 0,
  };
  const DEFAULT_LINK = {
    from: null,
    index: null,
    to: null,
  };
  const DEFAULT_OPTIONS = {
    draggable: true,
    pinchable: true,
    enable_grid: true,
    enable_filter: true,
    canvas: {
      scale: 1,
      grid: 10,
      grid_color: "#bbb",
    },
    node: {
      draggable: true,
      fit_grid: true,
      bgcolor: "#fff",
      border_color: "#007bff",
      border_width: 2,
      max_width: 300,
      font_size: {
        title: 16,
        body: 12,
      },
      default_text: {
        title: "No title",
        body: "No message",
      },
      text_color: {
        title: "#343a40",
        body: "#6c757d",
      },
    },
    link: {
      border_color: "#007bff",
      border_width: 1.5,
      type: 'line'
    },
    control: {
      detect_double_touch_msec: 500,
      detect_long_touch_msec: 500,
      detect_move_px: 10,
      drag_fps: 60,
    }
  };
  const LINK_TYPES = {
    line: 'line',
    key_line: 'key_line',
    curve: 'curve',
  };

  var options = {
    props: {
      options: {
        type: Object,
        default() {
          return DEFAULT_OPTIONS;
        },
      },
    },
    methods: {
      hasOption(name) {
        const names = name.split('.');
        const last = names.pop();

        return names.reduce((o, name) =>
          o.hasOwnProperty(name)
            ? o[name]
            : false
          , this.options || {}).hasOwnProperty(last);
      },
      to_i(v) {
        return parseInt(v, 10);
      },
      to_f(v, order) {
        const l = 10 ** order;
        return Math.round(parseFloat(v) * l) / l;
      },
      to_b(v) {
        return !!v;
      }

    },
    computed: {
      option_canvas_scale: {
        get() {
          return this.hasOption('canvas.scale')
            ? this.to_f(this.options.canvas.scale, 2)
            : DEFAULT_OPTIONS.canvas.scale;
        },
        set(scale) {
          if (this.hasOption('canvas.scale')) {
            return this.options.canvas.scale = scale;
          }
          if (this.hasOption('canvas')) {
            return this.options.canvas = Object.assign(this.options.canvas, { scale });
          }
          return this.options = Object.assign(this.options, { canvas: { scale } });
        }
      },
      option_draggable() {
        return this.hasOption('draggable')
          ? this.to_b(this.options.draggable)
          : DEFAULT_OPTIONS.draggable
      },
      option_pinchable() {
        return this.hasOption('pinchable')
          ? this.to_b(this.options.pinchable)
          : DEFAULT_OPTIONS.pinchable
      },
      option_enable_grid() {
        return this.hasOption('enable_grid')
          ? this.to_b(this.options.enable_grid)
          : DEFAULT_OPTIONS.enable_grid
      },
      option_enable_filter() {
        return this.hasOption('enable_filter')
          ? this.to_b(this.options.enable_filter)
          : DEFAULT_OPTIONS.enable_filter
      },
      option_canvas_grid() {
        return this.hasOption('canvas.grid')
          ? this.to_i(this.options.canvas.grid)
          : DEFAULT_OPTIONS.canvas.grid
      },
      option_canvas_grid_color() {
        return this.hasOption('canvas.grid_color')
          ? this.options.canvas.grid_color
          : DEFAULT_OPTIONS.canvas.grid_color
      },
      option_node_draggable() {
        return this.hasOption('node.draggable')
          ? this.to_b(this.options.node.draggable)
          : DEFAULT_OPTIONS.node.draggable
      },
      option_node_fit_grid() {
        return this.hasOption('node.fit_grid')
          ? this.to_b(this.options.node.fit_grid)
          : DEFAULT_OPTIONS.node.fit_grid
      },
      option_node_bgcolor() {
        return this.hasOption('node.bgcolor')
          ? this.options.node.bgcolor
          : DEFAULT_OPTIONS.node.bgcolor
      },
      option_node_border_color() {
        return this.hasOption('node.border_color')
          ? this.options.node.border_color
          : DEFAULT_OPTIONS.node.border_color
      },
      option_node_border_width() {
        return this.hasOption('node.border_width')
          ? this.to_i(this.options.node.border_width)
          : DEFAULT_OPTIONS.node.border_width
      },
      option_node_max_width() {
        return this.hasOption('node.max_width')
          ? this.to_i(this.options.node.max_width)
          : DEFAULT_OPTIONS.node.max_width
      },
      option_node_font_size_title() {
        return this.hasOption('node.font_size.title')
          ? this.to_i(this.options.node.font_size.title)
          : DEFAULT_OPTIONS.node.font_size.title
      },
      option_node_font_size_body() {
        return this.hasOption('node.font_size.body')
          ? this.to_i(this.options.node.font_size.body)
          : DEFAULT_OPTIONS.node.font_size.body
      },
      option_node_default_text_title() {
        return this.hasOption('node.default_text.title')
          ? this.options.node.default_text.title
          : DEFAULT_OPTIONS.node.default_text.title
      },
      option_node_default_text_body() {
        return this.hasOption('node.default_text.body')
          ? this.options.node.default_text.body
          : DEFAULT_OPTIONS.node.default_text.body
      },
      option_node_text_color_title() {
        return this.hasOption('node.text_color.title')
          ? this.options.node.text_color.title
          : DEFAULT_OPTIONS.node.text_color.title
      },
      option_node_text_color_body() {
        return this.hasOption('node.text_color.body')
          ? this.options.node.text_color.body
          : DEFAULT_OPTIONS.node.text_color.body
      },
      option_link_border_color() {
        return this.hasOption('link.border_color')
          ? this.options.link.border_color
          : DEFAULT_OPTIONS.link.border_color
      },
      option_link_border_width() {
        return this.hasOption('link.border_width')
          ? this.to_i(this.options.link.border_width)
          : DEFAULT_OPTIONS.link.border_width
      },
      option_link_type() {
        return this.hasOption('link.type')
          ? this.options.link.type
          : DEFAULT_OPTIONS.link.type
      },
      option_control_detect_double_touch_msec() {
        return this.hasOption('control.detect_double_touch_msec')
          ? this.to_i(this.options.control.detect_double_touch_msec)
          : DEFAULT_OPTIONS.control.detect_double_touch_msec
      },
      option_control_detect_long_touch_msec() {
        return this.hasOption('control.detect_long_touch_msec')
          ? this.to_i(this.options.control.detect_long_touch_msec)
          : DEFAULT_OPTIONS.control.detect_long_touch_msec
      },
      option_control_detect_move_px() {
        return this.hasOption('control.detect_move_px')
          ? this.to_i(this.options.control.detect_move_px)
          : DEFAULT_OPTIONS.control.detect_move_px
      },
      option_control_drag_fps() {
        return this.hasOption('control.drag_fps')
          ? this.to_i(this.options.control.drag_fps)
          : DEFAULT_OPTIONS.control.drag_fps
      },
    }
  };

  //
  var script = {
    name: "VueFlowchartConnection",
    mixins: [options],
    props: {
      connection: {
        type: Object,
        default() {
          return {
            fromX: 0,
            fromY: 0,
            toX: 0,
            toY: 0,
          };
        },
      },
    },
    computed: {
      path() {
        // https://developer.mozilla.org/ja/docs/Web/SVG/Attribute/d
        const [fx, fy] = [this.connection.fromX, this.connection.fromY];
        const [tx, ty] = [this.connection.toX, this.connection.toY];
        const l = 10;

        switch (this.option_link_type) {
          case LINK_TYPES.curve:
            return this.createCuve({ fx, fy, tx, ty, l });
          case LINK_TYPES.key_line:
            return this.createKeyLine({ fx, fy, tx, ty, l });
          default:
            return this.createLine({ fx, fy, tx, ty });
        }
      },
    },
    methods: {
      createLine({ fx, fy, tx, ty }) {
        return `M${fx},${fy} L${tx},${ty}`;
      },
      createKeyLine({ fx, fy, tx, ty, l }) {
        if (fx + l > tx) {
          return `M${fx},${fy} H${fx + l} V${(fy + ty) / 2} 
            H${tx - l} V${ty} H${tx}`;
        }
        return `M${fx},${fy} H${(fx + tx) / 2} V${ty} H${tx}`;
      },
      createCuve({ fx, fy, tx, ty, l }) {
        const [cx, cy] = [(fx + tx) / 2, (fy + ty) / 2];
        if (fx + l > tx) {
          return `M${fx},${fy} 
            Q${(fx + cx) / 2 + l * Math.sqrt(fx - cx)},${fy} ${cx},${cy}
            T${tx},${ty}`;
        }

        return `M${fx},${fy} 
          Q${(fx + cx) / 2},${fy} ${cx},${cy}
          T${tx},${ty}`;
      },
    },
  };

  function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
      if (typeof shadowMode !== 'boolean') {
          createInjectorSSR = createInjector;
          createInjector = shadowMode;
          shadowMode = false;
      }
      // Vue.extend constructor export interop.
      const options = typeof script === 'function' ? script.options : script;
      // render functions
      if (template && template.render) {
          options.render = template.render;
          options.staticRenderFns = template.staticRenderFns;
          options._compiled = true;
          // functional template
          if (isFunctionalTemplate) {
              options.functional = true;
          }
      }
      // scopedId
      if (scopeId) {
          options._scopeId = scopeId;
      }
      let hook;
      if (moduleIdentifier) {
          // server build
          hook = function (context) {
              // 2.3 injection
              context =
                  context || // cached call
                      (this.$vnode && this.$vnode.ssrContext) || // stateful
                      (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
              // 2.2 with runInNewContext: true
              if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                  context = __VUE_SSR_CONTEXT__;
              }
              // inject component styles
              if (style) {
                  style.call(this, createInjectorSSR(context));
              }
              // register component module identifier for async chunk inference
              if (context && context._registeredComponents) {
                  context._registeredComponents.add(moduleIdentifier);
              }
          };
          // used by ssr in case component is cached and beforeCreate
          // never gets called
          options._ssrRegister = hook;
      }
      else if (style) {
          hook = shadowMode
              ? function (context) {
                  style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
              }
              : function (context) {
                  style.call(this, createInjector(context));
              };
      }
      if (hook) {
          if (options.functional) {
              // register for functional component in vue file
              const originalRender = options.render;
              options.render = function renderWithStyleInjection(h, context) {
                  hook.call(context);
                  return originalRender(h, context);
              };
          }
          else {
              // inject component registration as beforeCreate hook
              const existing = options.beforeCreate;
              options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
          }
      }
      return script;
  }

  /* script */
  const __vue_script__ = script;

  /* template */
  var __vue_render__ = function() {
    var _vm = this;
    var _h = _vm.$createElement;
    var _c = _vm._self._c || _h;
    return _c("path", {
      staticClass: "vue-flowchart-svg-nodes-connections-connection",
      attrs: {
        d: _vm.path,
        "stroke-width": _vm.option_link_border_width,
        stroke: _vm.option_link_border_color,
        fill: "none",
        "stroke-linecap": "round"
      }
    })
  };
  var __vue_staticRenderFns__ = [];
  __vue_render__._withStripped = true;

    /* style */
    const __vue_inject_styles__ = undefined;
    /* scoped */
    const __vue_scope_id__ = undefined;
    /* module identifier */
    const __vue_module_identifier__ = undefined;
    /* functional template */
    const __vue_is_functional_template__ = false;
    /* style inject */
    
    /* style inject SSR */
    
    /* style inject shadow dom */
    

    
    const __vue_component__ = /*#__PURE__*/normalizeComponent(
      { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
      __vue_inject_styles__,
      __vue_script__,
      __vue_scope_id__,
      __vue_is_functional_template__,
      __vue_module_identifier__,
      false,
      undefined,
      undefined,
      undefined
    );

  //
  var script$1 = {
    name: "VueFlowchartNode",
    components: { Connection: __vue_component__ },
    mixins: [options],
    props: {
      is_selected: {
        type: Boolean,
        default: false,
      },
      node: {
        type: Object,
      },
      connections: {
        type: Array,
        default: () => {
          return [];
        },
      },
    },
    data() {
      return {
        width: 0,
        height: 0,
        hash: null,
      };
    },
    created() {
      this.width = this.node.w;
      this.height = this.node.h;
    },
    watch: {
      "node.h"(h) {
        this.height = h;
        this.$nextTick(() => this.updateNodeSize());
      },
      "node.bodies"(b) {
        this.$nextTick(() => this.updateNodeSize());
      },
      "options.node.max_width"(b) {
        this.$nextTick(() => this.updateNodeSize());
      },
    },
    mounted() {
      this.updateNodeSize();
    },
    methods: {
      toHash() {
        return [
          this.option_node_max_width,
          this.node.title,
          ...this.node.bodies,
        ].join("");
      },
      hashEquals(hash) {
        return this.hash && hash && this.hash === hash;
      },
      getBodyY(index) {
        return this.borderY + this.option_node_font_size_body * (1.5 + 2 * index);
      },
      updateNodeSize() {
        if (this.hashEquals(this.toHash())) {
          return;
        }
        this.hash = this.toHash();
        const padding = this.option_node_font_size_title;

        const title = this.$refs.title;
        title.innerHTML = this.node.title || this.option_node_default_text_title;

        const bodies = this.$refs.bodies || [];
        bodies.map(
          (b, i) =>
            (b.innerHTML =
              this.node.bodies[i] || this.option_node_default_text_body)
        );

        const max_text_width = Math.max(
          ...[title, ...bodies].map((n) => n.getComputedTextLength() + padding)
        );
        this.width = Math.min(max_text_width, this.option_node_max_width);

        if (this.width >= this.option_node_max_width) {
          this.ellipsisText();
        }
      },
      ellipsisText() {
        const padding = this.option_node_font_size_title;
        const title = this.$refs.title;
        const bodies = this.$refs.bodies || [];
        [title, ...bodies].map((n) => {
          let length = n.getComputedTextLength();
          let text = n.innerHTML;
          const padding = 5;
          while (length > this.width - 2 * padding && [...text].length > 0) {
            text = text.slice(0, -1);
            n.innerHTML = text + "...";
            length = n.getComputedTextLength();
          }
        });
      },
    },
    computed: {
      transform() {
        return `translate(${this.node.x}, ${this.node.y})`;
      },
      borderY() {
        return this.option_node_font_size_title * 2;
      },
      titleX() {
        return this.option_node_font_size_title / 2;
      },
      titleY() {
        return this.option_node_font_size_title * 1.5;
      },
      strokeWidth() {
        const width = this.option_node_border_width
          ? this.option_node_border_width
          : 1.5;
        return this.is_selected ? width * 2 : width;
      },
      computedConnections() {
        return this.connections.map((l) => {
          const fromX = this.width;
          const fromY =
            this.node.bodies.length === 1
              ? this.height / 2
              : (this.option_node_font_size_title +
                  (l.index + 0.5) * this.option_node_font_size_body) *
                2;

          const toX = l.to.x - this.node.x;
          const toY = l.to.y + l.to.h / 2 - this.node.y;

          return { fromX, fromY, toX, toY };
        });
      },
    },
  };

  const isOldIE = typeof navigator !== 'undefined' &&
      /msie [6-9]\\b/.test(navigator.userAgent.toLowerCase());
  function createInjector(context) {
      return (id, style) => addStyle(id, style);
  }
  let HEAD;
  const styles = {};
  function addStyle(id, css) {
      const group = isOldIE ? css.media || 'default' : id;
      const style = styles[group] || (styles[group] = { ids: new Set(), styles: [] });
      if (!style.ids.has(id)) {
          style.ids.add(id);
          let code = css.source;
          if (css.map) {
              // https://developer.chrome.com/devtools/docs/javascript-debugging
              // this makes source maps inside style tags work properly in Chrome
              code += '\n/*# sourceURL=' + css.map.sources[0] + ' */';
              // http://stackoverflow.com/a/26603875
              code +=
                  '\n/*# sourceMappingURL=data:application/json;base64,' +
                      btoa(unescape(encodeURIComponent(JSON.stringify(css.map)))) +
                      ' */';
          }
          if (!style.element) {
              style.element = document.createElement('style');
              style.element.type = 'text/css';
              if (css.media)
                  style.element.setAttribute('media', css.media);
              if (HEAD === undefined) {
                  HEAD = document.head || document.getElementsByTagName('head')[0];
              }
              HEAD.appendChild(style.element);
          }
          if ('styleSheet' in style.element) {
              style.styles.push(code);
              style.element.styleSheet.cssText = style.styles
                  .filter(Boolean)
                  .join('\n');
          }
          else {
              const index = style.ids.size - 1;
              const textNode = document.createTextNode(code);
              const nodes = style.element.childNodes;
              if (nodes[index])
                  style.element.removeChild(nodes[index]);
              if (nodes.length)
                  style.element.insertBefore(textNode, nodes[index]);
              else
                  style.element.appendChild(textNode);
          }
      }
  }

  /* script */
  const __vue_script__$1 = script$1;

  /* template */
  var __vue_render__$1 = function() {
    var _vm = this;
    var _h = _vm.$createElement;
    var _c = _vm._self._c || _h;
    return _c(
      "g",
      { attrs: { transform: _vm.transform } },
      [
        _c(
          "g",
          {
            staticClass: "vue-flowchart-svg-nodes-node",
            class: { selected: _vm.is_selected },
            attrs: {
              "stroke-width": _vm.strokeWidth,
              stroke: _vm.option_node_border_color,
              fill: _vm.option_node_bgcolor
            }
          },
          [
            _c("rect", {
              ref: "node",
              staticClass: "vue-flowchart-svg-nodes-node-box",
              attrs: {
                rx: "8",
                height: _vm.height,
                width: _vm.width,
                x: "0",
                y: "0"
              }
            }),
            _vm._v(" "),
            _c("line", {
              staticClass: "vue-flowchart-svg-nodes-node-border",
              attrs: { x1: "0", y1: _vm.borderY, x2: _vm.width, y2: _vm.borderY }
            }),
            _vm._v(" "),
            _c(
              "g",
              {
                staticClass: "vue-flowchart-svg-nodes-node-texts",
                attrs: { "text-anchor": "start", "stroke-width": "0" }
              },
              [
                _c(
                  "text",
                  {
                    ref: "title",
                    staticClass: "vue-flowchart-svg-nodes-node-texts-title",
                    attrs: {
                      x: _vm.titleX,
                      y: _vm.titleY,
                      "font-size": _vm.option_node_font_size_title,
                      fill: _vm.option_node_text_color_title,
                      "font-weight": "bold"
                    }
                  },
                  [
                    _vm._v(
                      _vm._s(_vm.node.title || _vm.option_node_default_text_title)
                    )
                  ]
                ),
                _vm._v(" "),
                _vm._l(_vm.node.bodies, function(body, index) {
                  return _c(
                    "text",
                    {
                      key: index,
                      ref: "bodies",
                      refInFor: true,
                      staticClass: "vue-flowchart-svg-nodes-node-texts-body",
                      attrs: {
                        x: _vm.titleX,
                        y: _vm.getBodyY(index),
                        fill: _vm.option_node_text_color_body,
                        "font-size": _vm.option_node_font_size_body
                      }
                    },
                    [_vm._v(_vm._s(body || _vm.option_node_default_text_body))]
                  )
                })
              ],
              2
            )
          ]
        ),
        _vm._v(" "),
        _vm._l(_vm.computedConnections, function(connection, index) {
          return _c(
            "g",
            { key: index, staticClass: "vue-flowchart-svg-nodes-connections" },
            [
              _c("Connection", {
                attrs: { connection: connection, options: _vm.options }
              })
            ],
            1
          )
        })
      ],
      2
    )
  };
  var __vue_staticRenderFns__$1 = [];
  __vue_render__$1._withStripped = true;

    /* style */
    const __vue_inject_styles__$1 = function (inject) {
      if (!inject) return
      inject("data-v-6347c346_0", { source: ".vue-flowchart-svg-nodes-node[data-v-6347c346] {\n  user-select: none;\n  cursor: pointer;\n}\n.vue-flowchart-svg-nodes-node.selected[data-v-6347c346] {\n  cursor: move;\n}\n\n/*# sourceMappingURL=vue-flowchart-node.vue.map */", map: {"version":3,"sources":["Z:\\dev\\vue-flowchart\\package\\src\\components\\vue-flowchart-node.vue","vue-flowchart-node.vue"],"names":[],"mappings":"AA6MA;EACA,iBAAA;EACA,eAAA;AC5MA;AD8MA;EACA,YAAA;AC5MA;;AAEA,iDAAiD","file":"vue-flowchart-node.vue","sourcesContent":["<template>\r\n  <g :transform=\"transform\">\r\n    <g\r\n      class=\"vue-flowchart-svg-nodes-node\"\r\n      :class=\"{ selected: is_selected }\"\r\n      :stroke-width=\"strokeWidth\"\r\n      :stroke=\"option_node_border_color\"\r\n      :fill=\"option_node_bgcolor\"\r\n    >\r\n      <rect\r\n        class=\"vue-flowchart-svg-nodes-node-box\"\r\n        rx=\"8\"\r\n        :height=\"height\"\r\n        :width=\"width\"\r\n        x=\"0\"\r\n        y=\"0\"\r\n        ref=\"node\"\r\n      />\r\n      <line\r\n        class=\"vue-flowchart-svg-nodes-node-border\"\r\n        x1=\"0\"\r\n        :y1=\"borderY\"\r\n        :x2=\"width\"\r\n        :y2=\"borderY\"\r\n      />\r\n      <g\r\n        class=\"vue-flowchart-svg-nodes-node-texts\"\r\n        text-anchor=\"start\"\r\n        stroke-width=\"0\"\r\n      >\r\n        <text\r\n          class=\"vue-flowchart-svg-nodes-node-texts-title\"\r\n          :x=\"titleX\"\r\n          :y=\"titleY\"\r\n          :font-size=\"option_node_font_size_title\"\r\n          :fill=\"option_node_text_color_title\"\r\n          font-weight=\"bold\"\r\n          ref=\"title\"\r\n          >{{ node.title || option_node_default_text_title }}</text\r\n        >\r\n        <text\r\n          class=\"vue-flowchart-svg-nodes-node-texts-body\"\r\n          v-for=\"(body, index) in node.bodies\"\r\n          :key=\"index\"\r\n          :x=\"titleX\"\r\n          :y=\"getBodyY(index)\"\r\n          :fill=\"option_node_text_color_body\"\r\n          :font-size=\"option_node_font_size_body\"\r\n          ref=\"bodies\"\r\n          >{{ body || option_node_default_text_body }}</text\r\n        >\r\n      </g>\r\n    </g>\r\n    <g\r\n      v-for=\"(connection, index) in computedConnections\"\r\n      :key=\"index\"\r\n      class=\"vue-flowchart-svg-nodes-connections\"\r\n    >\r\n      <Connection :connection=\"connection\" :options=\"options\" />\r\n    </g>\r\n  </g>\r\n</template>\r\n<script>\r\nimport Connection from \"./vue-flowchart-connection.vue\";\r\nimport options from \"../options\";\r\nexport default {\r\n  name: \"VueFlowchartNode\",\r\n  components: { Connection },\r\n  mixins: [options],\r\n  props: {\r\n    is_selected: {\r\n      type: Boolean,\r\n      default: false,\r\n    },\r\n    node: {\r\n      type: Object,\r\n    },\r\n    connections: {\r\n      type: Array,\r\n      default: () => {\r\n        return [];\r\n      },\r\n    },\r\n  },\r\n  data() {\r\n    return {\r\n      width: 0,\r\n      height: 0,\r\n      hash: null,\r\n    };\r\n  },\r\n  created() {\r\n    this.width = this.node.w;\r\n    this.height = this.node.h;\r\n  },\r\n  watch: {\r\n    \"node.h\"(h) {\r\n      this.height = h;\r\n      this.$nextTick(() => this.updateNodeSize());\r\n    },\r\n    \"node.bodies\"(b) {\r\n      this.$nextTick(() => this.updateNodeSize());\r\n    },\r\n    \"options.node.max_width\"(b) {\r\n      this.$nextTick(() => this.updateNodeSize());\r\n    },\r\n  },\r\n  mounted() {\r\n    this.updateNodeSize();\r\n  },\r\n  methods: {\r\n    toHash() {\r\n      return [\r\n        this.option_node_max_width,\r\n        this.node.title,\r\n        ...this.node.bodies,\r\n      ].join(\"\");\r\n    },\r\n    hashEquals(hash) {\r\n      return this.hash && hash && this.hash === hash;\r\n    },\r\n    getBodyY(index) {\r\n      return this.borderY + this.option_node_font_size_body * (1.5 + 2 * index);\r\n    },\r\n    updateNodeSize() {\r\n      if (this.hashEquals(this.toHash())) {\r\n        return;\r\n      }\r\n      this.hash = this.toHash();\r\n      const padding = this.option_node_font_size_title;\r\n\r\n      const title = this.$refs.title;\r\n      title.innerHTML = this.node.title || this.option_node_default_text_title;\r\n\r\n      const bodies = this.$refs.bodies || [];\r\n      bodies.map(\r\n        (b, i) =>\r\n          (b.innerHTML =\r\n            this.node.bodies[i] || this.option_node_default_text_body)\r\n      );\r\n\r\n      const max_text_width = Math.max(\r\n        ...[title, ...bodies].map((n) => n.getComputedTextLength() + padding)\r\n      );\r\n      this.width = Math.min(max_text_width, this.option_node_max_width);\r\n\r\n      if (this.width >= this.option_node_max_width) {\r\n        this.ellipsisText();\r\n      }\r\n    },\r\n    ellipsisText() {\r\n      const padding = this.option_node_font_size_title;\r\n      const title = this.$refs.title;\r\n      const bodies = this.$refs.bodies || [];\r\n      [title, ...bodies].map((n) => {\r\n        let length = n.getComputedTextLength();\r\n        let text = n.innerHTML;\r\n        const padding = 5;\r\n        while (length > this.width - 2 * padding && [...text].length > 0) {\r\n          text = text.slice(0, -1);\r\n          n.innerHTML = text + \"...\";\r\n          length = n.getComputedTextLength();\r\n        }\r\n      });\r\n    },\r\n  },\r\n  computed: {\r\n    transform() {\r\n      return `translate(${this.node.x}, ${this.node.y})`;\r\n    },\r\n    borderY() {\r\n      return this.option_node_font_size_title * 2;\r\n    },\r\n    titleX() {\r\n      return this.option_node_font_size_title / 2;\r\n    },\r\n    titleY() {\r\n      return this.option_node_font_size_title * 1.5;\r\n    },\r\n    strokeWidth() {\r\n      const width = this.option_node_border_width\r\n        ? this.option_node_border_width\r\n        : 1.5;\r\n      return this.is_selected ? width * 2 : width;\r\n    },\r\n    computedConnections() {\r\n      return this.connections.map((l) => {\r\n        const fromX = this.width;\r\n        const fromY =\r\n          this.node.bodies.length === 1\r\n            ? this.height / 2\r\n            : (this.option_node_font_size_title +\r\n                (l.index + 0.5) * this.option_node_font_size_body) *\r\n              2;\r\n\r\n        const toX = l.to.x - this.node.x;\r\n        const toY = l.to.y + l.to.h / 2 - this.node.y;\r\n\r\n        return { fromX, fromY, toX, toY };\r\n      });\r\n    },\r\n  },\r\n};\r\n</script>\r\n<style lang=\"scss\" scoped>\r\n.vue-flowchart-svg-nodes-node {\r\n  user-select: none;\r\n  cursor: pointer;\r\n\r\n  &.selected {\r\n    cursor: move;\r\n  }\r\n}\r\n</style>",".vue-flowchart-svg-nodes-node {\n  user-select: none;\n  cursor: pointer;\n}\n.vue-flowchart-svg-nodes-node.selected {\n  cursor: move;\n}\n\n/*# sourceMappingURL=vue-flowchart-node.vue.map */"]}, media: undefined });

    };
    /* scoped */
    const __vue_scope_id__$1 = "data-v-6347c346";
    /* module identifier */
    const __vue_module_identifier__$1 = undefined;
    /* functional template */
    const __vue_is_functional_template__$1 = false;
    /* style inject SSR */
    
    /* style inject shadow dom */
    

    
    const __vue_component__$1 = /*#__PURE__*/normalizeComponent(
      { render: __vue_render__$1, staticRenderFns: __vue_staticRenderFns__$1 },
      __vue_inject_styles__$1,
      __vue_script__$1,
      __vue_scope_id__$1,
      __vue_is_functional_template__$1,
      __vue_module_identifier__$1,
      false,
      createInjector,
      undefined,
      undefined
    );

  var handler = {
    data() {
      return {
        touches: [],
        touch_node: null,   // touch target is node or background
        touched_at: null,   // for double touch
        touch_timer: null,  // for long touch
        offset: { x: 0, y: 0 },
        render_area: { w: 100, h: 100 },
      };
    },
    methods: {
      // touch start
      handleTouchStart(event, node) {
        this.processStart([...event.touches], node);
      },
      handleMouseDown(event, node) {
        this.processStart([event], node);
      },
      processStart(touches, node) {
        const touched_at = new Date();
        const node_id = node ? node.id : null;

        this.touch_node = node_id;
        if (this.option_draggable || this.option_node_draggable || this.option_pinchable) {
          this.addMoveListener();
        }

        if (this.touch_timer) {
          clearTimeout(this.touch_timer);
          this.touch_timer = null;
        }

        if (this.isDoubleTouch(touches, touched_at)) {
          this.$emit('onDoubleTouch', { node_id });
          return;
        }

        this.touch_timer = setTimeout(() => {
          if (this.isLongTouch(touches)) {
            this.$emit('onLongTouch', { node_id });
            this.removeMoveListener();
          }
        }, this.option_control_detect_long_touch_msec);

        this.touches = touches;
        this.touched_at = touched_at;
        this.$emit('onTouch', { node_id });
      },

      // touch end
      handleTouchEnd(event, node) {
        this.processEnd([...event.touches], node);
      },
      handleMouseUp(event, node) {
        this.processEnd([], node);
      },
      processEnd(touches, node) {
        if (this.touch_timer) {
          clearTimeout(this.touch_timer);
          this.touch_timer = null;
        }

        if (touches.length === 0) {
          this.removeMoveListener();
          const node_id = node ? node.id : null;
          this.$emit('onTouchEnd', { node_id });
        }
      },

      // mouve handling
      addMoveListener() {
        if (this.option_draggable || this.option_pinchable || this.option_node_draggable) {
          this.$el.addEventListener('touchmove', this.handleTouchMove, { passive: false });
          this.$el.addEventListener('mousemove', this.handleMouseMove, { passive: true });
          this.$el.addEventListener('wheel', this.handleWheel, { passive: false });
        }
      },
      removeMoveListener() {
        this.$el.removeEventListener('touchmove', this.handleTouchMove);
        this.$el.removeEventListener('mousemove', this.handleMouseMove);
        this.$el.removeEventListener('wheel', this.handleWheel);
      },
      handleTouchMove(event) {
        event.preventDefault();
        this.processMove([...event.touches]);
      },
      handleMouseMove(event) {
        this.processMove([event]);
      },
      limitFPS() {
        if (this.option_control_drag_fps < 120) {
          this.removeMoveListener();
          setTimeout(() => {
            if (this.touch_timer) {
              this.addMoveListener();
            }
          }, 1000 / this.option_control_drag_fps);
        }
      },
      processMove(touches) {
        this.limitFPS();

        if (this.isDrag(touches)) {
          if (!this.shouldMove(touches[0])) {
            return;
          }
          const { dx, dy } = this.calcDrag(touches[0]);

          this.touches = touches;
          if (this.option_node_draggable && this.touch_node) {
            this.$emit('onDrag', { dx, dy });
            return this.updateNodePosition({ dx, dy })
          }
          if (this.option_draggable) {
            this.$emit('onBackgroundDrag', { dx, dy });
            return this.updateBackgroundPosition({ dx, dy })
          }
        }
        if (this.option_pinchable && this.isPinch(touches)) {
          const { dr, dh, dv, dl, cx, cy } = this.calcPinch(touches);

          this.touches = touches;
          this.$emit('onPinch', { dr, dh, dv, dl, cx, cy });
          this.updateScale({ dl, cx, cy });
        }
      },
      handleWheel(event) {
        event.preventDefault();

        if (this.touch_timer) {
          clearTimeout(this.touch_timer);
        }

        if (event.deltaY) {
          this.updateScale({ dl: -event.deltaY, cx: event.clientX, cy: event.clientY });
        }
      },

      updateNodePosition({ dx, dy }) {
        const [real_dx, real_dy] = [dx / this.option_canvas_scale, dy / this.option_canvas_scale];
        this.flow.nodes = this.flow.nodes.map((n) => {
          if (this.selected_node_ids.includes(n.id)) {
            const [x, y] = this.option_node_fit_grid
              ? [
                Math.round((n.x + real_dx) / this.option_canvas_grid) * this.option_canvas_grid,
                Math.round((n.y + real_dy) / this.option_canvas_grid) * this.option_canvas_grid
              ]
              : [
                n.x + real_dx, n.y + real_dy
              ];
            return Object.assign({}, n, { x: this.to_i(x), y: this.to_i(y) });
          }
          return n;
        }
        );
      },
      updateBackgroundPosition({ dx, dy }) {
        const [real_dx, real_dy] = [dx / this.option_canvas_scale, dy / this.option_canvas_scale];
        this.offset.x += this.to_i(real_dx);
        this.offset.y += this.to_i(real_dy);
      },
      updateScale({ dl, cx, cy }) {
        const old_scale = this.option_canvas_scale;
        const new_scale = old_scale + dl / this.render_area.w;
        if (new_scale <= 0.2 || 5 < new_scale) {
          return;
        }

        this.offset.x = this.to_i(this.offset.x - (new_scale - old_scale) * cx);
        this.offset.y = this.to_i(this.offset.y - (new_scale - old_scale) * cy);
        this.option_canvas_scale = this.to_f(new_scale, 2);
      },
      updateRenderArea() {
        this.render_area.w = this.$el.offsetWidth;
        this.render_area.h = this.$el.offsetHeight;
      },
      isDoubleTouch(touches, touched_at) {
        if (this.touched_at
          && touched_at - this.touched_at < this.option_control_detect_double_touch_msec
          && this.touches.length === 1 && touches.length === 1) {
          const t0 = this.touches[0];
          const t1 = touches[0];
          return Math.abs(t0.clientX - t1.clientX) < this.option_control_detect_move_px
            && Math.abs(t0.clientY - t1.clientY) < this.option_control_detect_move_px;
        }
        return false;
      },
      isLongTouch(touches) {
        if (this.touches.length === 1 && touches.length === 1) {
          const t0 = this.touches[0];
          const t1 = touches[0];
          return Math.abs(t0.clientX - t1.clientX) < this.option_control_detect_move_px
            && Math.abs(t0.clientY - t1.clientY) < this.option_control_detect_move_px;
        }
        return false;
      },
      isDrag(touches) {
        if (this.touches.length === 1 && touches.length === 1) {
          const t0 = this.touches[0];
          const t1 = touches[0];
          return Math.abs(t0.clientX - t1.clientX) > this.option_control_detect_move_px
            || Math.abs(t0.clientY - t1.clientY) > this.option_control_detect_move_px;
        }
        return false;
      },
      shouldMove(touch) {
        if (!this.option_node_fit_grid) {
          return true;
        }
        const t0 = this.touches[0];
        const t1 = touch;
        return Math.abs(t0.clientX - t1.clientX) >= this.option_canvas_grid
          || Math.abs(t0.clientY - t1.clientY) >= this.option_canvas_grid;
      },
      isPinch(touches) {
        if (this.touches.length >= 2 && touches.length >= 2) {
          const [t00, t10] = [this.touches[0], this.touches[1]];
          const [t01, t11] = [touches[0], touches[1]];
          return Math.abs(Math.abs(t00.clientX - t10.clientX) - Math.abs(t01.clientX - t11.clientX)) > this.option_control_detect_move_px
            || Math.abs(Math.abs(t00.clientY - t10.clientY) - Math.abs(t01.clientY - t11.clientY)) > this.option_control_detect_move_px
        }
        return false;
      },
      calcDrag(touch) {
        const dx = touch.clientX - this.touches[0].clientX;
        const dy = touch.clientY - this.touches[0].clientY;

        return { dx, dy };
      },
      calcPinch(touches) {
        // distance
        const w1 = this.touches[0].clientX - this.touches[1].clientX;
        const h1 = this.touches[0].clientY - this.touches[1].clientY;
        const w2 = touches[0].clientX - touches[1].clientX;
        const h2 = touches[0].clientY - touches[1].clientY;
        // center
        const cx = (touches[0].clientX + touches[1].clientX) / 2;
        const cy = (touches[0].clientY + touches[1].clientY) / 2;

        // rotate
        const r1 = Math.abs(Math.atan(h1 / w1));
        const r2 = Math.abs(Math.atan(h2 / w2));
        const dr = (r2 - r1) * 180 / Math.PI;
        // horizontal
        const dh = Math.abs(h2) - Math.abs(h1);
        // vertical
        const dv = Math.abs(w2) - Math.abs(w1);

        // length
        const dl = Math.sqrt(w2 ** 2 + h2 ** 2) - Math.sqrt(w1 ** 2 + h1 ** 2);

        return { dr, dh, dv, dl, cx, cy }
      },
    },
    computed: {
      boxStyle() {
        return `max-width: ${this.render_area.w}px; margin: auto;`;
      },
      transform() {
        return `scale(${this.option_canvas_scale}) translate(${this.offset.x}, ${this.offset.y}) `;
      },
      viewbox() {
        return `0 0 ${this.render_area.w} ${this.render_area.h}`;
      },
      gridTransform() {
        const x = (this.offset.x % this.option_canvas_grid) - this.option_canvas_grid;
        const y = (this.offset.y % this.option_canvas_grid) - this.option_canvas_grid;
        return `scale(${this.option_canvas_scale}) translate(${x}, ${y}) `;
      },
      gridSize() {
        return this.option_canvas_grid * this.option_canvas_scale;
      },
      gridPoints() {
        return `0, ${this.option_canvas_grid} 0, 0 ${this.option_canvas_grid}, 0`
      },
      gridCount() {
        return {
          x: 2 + Math.round(this.render_area.w / this.gridSize),
          y: 2 + Math.round(this.render_area.h / this.gridSize)
        };
      },
      gridArea() {
        return {
          w: this.gridCount.x * this.option_canvas_grid,
          h: this.gridCount.y * this.option_canvas_grid,
        }
      },
      gridScale() {
        return {
          x: 1 / this.gridCount.x,
          y: 1 / this.gridCount.y
        }
      }
    },
    mounted() {
      this.$el.addEventListener('resize', this.updateRenderArea);
      this.updateRenderArea();
    }
  };

  //
  var script$2 = {
    name: "VueFlowchart",
    components: { Node: __vue_component__$1 },
    mixins: [handler, options],
    props: {
      id: {
        type: String,
        default: "vue-flowchart",
      },
      flow: {
        type: Object,
        default() {
          return {
            nodes: [],
            links: [],
          };
        },
      },
      selected_node_ids: {
        type: Array,
        default() {
          return [];
        },
      },
    },
    methods: {
      /**
       * 内容に対して高さが足りなければ内容から算出した高さにする
       */
      calculateHeight(node) {
        const bodies = node.bodies.length ? node.bodies.length : 1;

        return (
          this.option_node_font_size_body * bodies * 2 +
          this.option_node_font_size_title * 2
        );
      },
      isSelected(node) {
        return this.selected_node_ids.includes(node.id);
      },
      getConnections(node) {
        return this.computedLinks
          .filter((l) => l.from === node.id)
          .map((l) => {
            return {
              index: l.index,
              to: this.computedNodes.find((n) => n.id === l.to),
            };
          });
      },
    },
    computed: {
      computedFlow() {
        return Object.assign({ nodes: [], links: [] }, this.flow);
      },
      computedNodes() {
        return this.computedFlow.nodes
          .map((n) => Object.assign({}, DEFAULT_NODE, n))
          .map((n) =>
            Object.assign(n, {
              w: this.option_node_max_width,
              h: this.calculateHeight(n),
            })
          );
      },
      computedLinks() {
        return this.computedFlow.links.map((l) =>
          Object.assign({}, DEFAULT_LINK, l)
        );
      },
    },
  };

  /* script */
  const __vue_script__$2 = script$2;

  /* template */
  var __vue_render__$2 = function() {
    var _vm = this;
    var _h = _vm.$createElement;
    var _c = _vm._self._c || _h;
    return _c("div", { staticClass: "vue-flowchart" }, [
      _c(
        "svg",
        {
          ref: "svg",
          staticClass: "vue-flowchart-svg",
          attrs: {
            xmlns: "http://www.w3.org/2000/svg",
            id: _vm.id,
            height: _vm.render_area.h,
            width: _vm.render_area.w,
            viewBox: _vm.viewbox,
            overflow: "hidden"
          },
          on: {
            touchstart: function($event) {
              $event.stopPropagation();
              $event.preventDefault();
              return (function(e) {
                return _vm.handleTouchStart(e, null)
              })($event)
            },
            mousedown: function($event) {
              $event.stopPropagation();
              $event.preventDefault();
              return (function(e) {
                return _vm.handleMouseDown(e, null)
              })($event)
            },
            touchend: function($event) {
              $event.stopPropagation();
              $event.preventDefault();
              return (function(e) {
                return _vm.handleTouchEnd(e, null)
              })($event)
            },
            mouseup: function($event) {
              $event.stopPropagation();
              $event.preventDefault();
              return (function(e) {
                return _vm.handleMouseUp(e, null)
              })($event)
            }
          }
        },
        [
          _vm.option_enable_grid
            ? _vm._t(
                "grid",
                [
                  _c("defs", [
                    _c(
                      "pattern",
                      {
                        attrs: {
                          id: "grid",
                          x: "0",
                          y: "0",
                          width: _vm.gridScale.x,
                          height: _vm.gridScale.y
                        }
                      },
                      [
                        _c("polyline", {
                          attrs: {
                            points: _vm.gridPoints,
                            stroke: _vm.option_canvas_grid_color,
                            "stroke-width": "1",
                            fill: "none"
                          }
                        })
                      ]
                    )
                  ])
                ],
                { options: _vm.options, gridScale: _vm.gridScale }
              )
            : _vm._e(),
          _vm._v(" "),
          _c(
            "g",
            { staticClass: "grid", attrs: { transform: _vm.gridTransform } },
            [
              _c("rect", {
                staticClass: "vue-flowchart-svg-grid",
                attrs: {
                  fill: "url(#grid)",
                  x: "0",
                  y: "0",
                  width: _vm.gridArea.w,
                  height: _vm.gridArea.h
                }
              })
            ]
          ),
          _vm._v(" "),
          _c(
            "g",
            {
              staticClass: "vue-flowchart-svg-nodes",
              attrs: {
                filter: _vm.option_enable_filter ? "url(#filter)" : "none",
                transform: _vm.transform
              }
            },
            _vm._l(_vm.computedNodes, function(node) {
              return _c(
                "g",
                {
                  key: node.id,
                  on: {
                    touchstart: function($event) {
                      $event.stopPropagation();
                      $event.preventDefault();
                      return (function(e) {
                        return _vm.handleTouchStart(e, node)
                      })($event)
                    },
                    mousedown: function($event) {
                      $event.stopPropagation();
                      $event.preventDefault();
                      return (function(e) {
                        return _vm.handleMouseDown(e, node)
                      })($event)
                    },
                    touchend: function($event) {
                      $event.stopPropagation();
                      $event.preventDefault();
                      return (function(e) {
                        return _vm.handleTouchEnd(e, node)
                      })($event)
                    },
                    mouseup: function($event) {
                      $event.stopPropagation();
                      $event.preventDefault();
                      return (function(e) {
                        return _vm.handleMouseUp(e, node)
                      })($event)
                    }
                  }
                },
                [
                  _c("Node", {
                    attrs: {
                      node: node,
                      connections: _vm.getConnections(node),
                      options: _vm.options,
                      is_selected: _vm.isSelected(node)
                    }
                  })
                ],
                1
              )
            }),
            0
          ),
          _vm._v(" "),
          _vm._t("filter", [
            _c("defs", [
              _c(
                "filter",
                { attrs: { id: "filter" } },
                [
                  _c("feGaussianBlur", {
                    attrs: {
                      in: "SourceAlpha",
                      stdDeviation: "2",
                      result: "blur"
                    }
                  }),
                  _vm._v(" "),
                  _c(
                    "feComponentTransfer",
                    { attrs: { in: "blur", result: "blur_opacity" } },
                    [_c("feFuncA", { attrs: { type: "linear", slope: "0.25" } })],
                    1
                  ),
                  _vm._v(" "),
                  _c("feOffset", {
                    attrs: { in: "linear", dx: "1", dy: "1", result: "offset" }
                  }),
                  _vm._v(" "),
                  _c(
                    "feMerge",
                    [
                      _c("feMergeNode", { attrs: { in: "offset" } }),
                      _vm._v(" "),
                      _c("feMergeNode", { attrs: { in: "SourceGraphic" } })
                    ],
                    1
                  )
                ],
                1
              )
            ])
          ])
        ],
        2
      )
    ])
  };
  var __vue_staticRenderFns__$2 = [];
  __vue_render__$2._withStripped = true;

    /* style */
    const __vue_inject_styles__$2 = function (inject) {
      if (!inject) return
      inject("data-v-b809ea58_0", { source: ".vue-flowchart[data-v-b809ea58] {\n  overflow: hidden;\n  border: solid 1px #000;\n}\n\n/*# sourceMappingURL=vue-flowchart.vue.map */", map: {"version":3,"sources":["Z:\\dev\\vue-flowchart\\package\\src\\vue-flowchart.vue","vue-flowchart.vue"],"names":[],"mappings":"AAwKA;EACA,gBAAA;EACA,sBAAA;ACvKA;;AAEA,4CAA4C","file":"vue-flowchart.vue","sourcesContent":["<template>\r\n  <div class=\"vue-flowchart\">\r\n    <svg\r\n      class=\"vue-flowchart-svg\"\r\n      xmlns=\"http://www.w3.org/2000/svg\"\r\n      ref=\"svg\"\r\n      :id=\"id\"\r\n      :height=\"render_area.h\"\r\n      :width=\"render_area.w\"\r\n      :viewBox=\"viewbox\"\r\n      overflow=\"hidden\"\r\n      @touchstart.stop.prevent=\"(e) => handleTouchStart(e, null)\"\r\n      @mousedown.stop.prevent=\"(e) => handleMouseDown(e, null)\"\r\n      @touchend.stop.prevent=\"(e) => handleTouchEnd(e, null)\"\r\n      @mouseup.stop.prevent=\"(e) => handleMouseUp(e, null)\"\r\n    >\r\n      <slot\r\n        name=\"grid\"\r\n        v-if=\"option_enable_grid\"\r\n        :options=\"options\"\r\n        :gridScale=\"gridScale\"\r\n      >\r\n        <defs>\r\n          <pattern\r\n            id=\"grid\"\r\n            x=\"0\"\r\n            y=\"0\"\r\n            :width=\"gridScale.x\"\r\n            :height=\"gridScale.y\"\r\n          >\r\n            <polyline\r\n              :points=\"gridPoints\"\r\n              :stroke=\"option_canvas_grid_color\"\r\n              stroke-width=\"1\"\r\n              fill=\"none\"\r\n            />\r\n          </pattern>\r\n        </defs>\r\n      </slot>\r\n      <g class=\"grid\" :transform=\"gridTransform\">\r\n        <rect\r\n          class=\"vue-flowchart-svg-grid\"\r\n          fill=\"url(#grid)\"\r\n          x=\"0\"\r\n          y=\"0\"\r\n          :width=\"gridArea.w\"\r\n          :height=\"gridArea.h\"\r\n        />\r\n      </g>\r\n      <g\r\n        class=\"vue-flowchart-svg-nodes\"\r\n        :filter=\"option_enable_filter ? 'url(#filter)' : 'none'\"\r\n        :transform=\"transform\"\r\n      >\r\n        <g\r\n          v-for=\"node in computedNodes\"\r\n          :key=\"node.id\"\r\n          @touchstart.stop.prevent=\"(e) => handleTouchStart(e, node)\"\r\n          @mousedown.stop.prevent=\"(e) => handleMouseDown(e, node)\"\r\n          @touchend.stop.prevent=\"(e) => handleTouchEnd(e, node)\"\r\n          @mouseup.stop.prevent=\"(e) => handleMouseUp(e, node)\"\r\n        >\r\n          <Node\r\n            :node=\"node\"\r\n            :connections=\"getConnections(node)\"\r\n            :options=\"options\"\r\n            :is_selected=\"isSelected(node)\"\r\n          />\r\n        </g>\r\n      </g>\r\n      <slot name=\"filter\">\r\n        <defs>\r\n          <filter id=\"filter\">\r\n            <feGaussianBlur in=\"SourceAlpha\" stdDeviation=\"2\" result=\"blur\" />\r\n            <feComponentTransfer in=\"blur\" result=\"blur_opacity\">\r\n              <feFuncA type=\"linear\" slope=\"0.25\" />\r\n            </feComponentTransfer>\r\n            <feOffset in=\"linear\" dx=\"1\" dy=\"1\" result=\"offset\" />\r\n            <feMerge>\r\n              <feMergeNode in=\"offset\" />\r\n              <feMergeNode in=\"SourceGraphic\" />\r\n            </feMerge>\r\n          </filter>\r\n        </defs>\r\n      </slot>\r\n    </svg>\r\n  </div>\r\n</template>\r\n\r\n<script>\r\nimport Node from \"./components/vue-flowchart-node.vue\";\r\nimport handler from \"./handler\";\r\nimport options from \"./options\";\r\nimport { DEFAULT_NODE, DEFAULT_LINK } from \"./const\";\r\nexport default {\r\n  name: \"VueFlowchart\",\r\n  components: { Node },\r\n  mixins: [handler, options],\r\n  props: {\r\n    id: {\r\n      type: String,\r\n      default: \"vue-flowchart\",\r\n    },\r\n    flow: {\r\n      type: Object,\r\n      default() {\r\n        return {\r\n          nodes: [],\r\n          links: [],\r\n        };\r\n      },\r\n    },\r\n    selected_node_ids: {\r\n      type: Array,\r\n      default() {\r\n        return [];\r\n      },\r\n    },\r\n  },\r\n  methods: {\r\n    /**\r\n     * 内容に対して高さが足りなければ内容から算出した高さにする\r\n     */\r\n    calculateHeight(node) {\r\n      const bodies = node.bodies.length ? node.bodies.length : 1;\r\n\r\n      return (\r\n        this.option_node_font_size_body * bodies * 2 +\r\n        this.option_node_font_size_title * 2\r\n      );\r\n    },\r\n    isSelected(node) {\r\n      return this.selected_node_ids.includes(node.id);\r\n    },\r\n    getConnections(node) {\r\n      return this.computedLinks\r\n        .filter((l) => l.from === node.id)\r\n        .map((l) => {\r\n          return {\r\n            index: l.index,\r\n            to: this.computedNodes.find((n) => n.id === l.to),\r\n          };\r\n        });\r\n    },\r\n  },\r\n  computed: {\r\n    computedFlow() {\r\n      return Object.assign({ nodes: [], links: [] }, this.flow);\r\n    },\r\n    computedNodes() {\r\n      return this.computedFlow.nodes\r\n        .map((n) => Object.assign({}, DEFAULT_NODE, n))\r\n        .map((n) =>\r\n          Object.assign(n, {\r\n            w: this.option_node_max_width,\r\n            h: this.calculateHeight(n),\r\n          })\r\n        );\r\n    },\r\n    computedLinks() {\r\n      return this.computedFlow.links.map((l) =>\r\n        Object.assign({}, DEFAULT_LINK, l)\r\n      );\r\n    },\r\n  },\r\n};\r\n</script>\r\n<style lang=\"scss\" scoped>\r\n.vue-flowchart {\r\n  overflow: hidden;\r\n  border: solid 1px #000;\r\n}\r\n</style>",".vue-flowchart {\n  overflow: hidden;\n  border: solid 1px #000;\n}\n\n/*# sourceMappingURL=vue-flowchart.vue.map */"]}, media: undefined });

    };
    /* scoped */
    const __vue_scope_id__$2 = "data-v-b809ea58";
    /* module identifier */
    const __vue_module_identifier__$2 = undefined;
    /* functional template */
    const __vue_is_functional_template__$2 = false;
    /* style inject SSR */
    
    /* style inject shadow dom */
    

    
    const __vue_component__$2 = /*#__PURE__*/normalizeComponent(
      { render: __vue_render__$2, staticRenderFns: __vue_staticRenderFns__$2 },
      __vue_inject_styles__$2,
      __vue_script__$2,
      __vue_scope_id__$2,
      __vue_is_functional_template__$2,
      __vue_module_identifier__$2,
      false,
      createInjector,
      undefined,
      undefined
    );

  function install(Vue) {
    if (install.installed) return;
    install.installed = true;
    Vue.component('VueFlowchart', __vue_component__$2);
  }

  const plugin = {
    install,
  };

  let GlobalVue = null;
  if (typeof window !== 'undefined') {
    GlobalVue = window.Vue;
  } else if (typeof global !== 'undefined') {
    GlobalVue = global.Vue;
  }
  if (GlobalVue) {
    GlobalVue.use(plugin);
  }
  __vue_component__$2.install = install;

  exports.default = __vue_component__$2;
  exports.install = install;

  return exports;

}({}));
